import requests
from urllib.parse import urljoin
from sync_amz_data.public.amz_ad_client import SBClient
from sync_amz_data.settings import AWS_LWA_CLIENT
import pandas as pd
import json
from sync_amz_data.tasks.datainsert.wg import LADS


class RateLimitError(Exception):
    def __init__(self, retry_after: str = None):
        self.retry_after = retry_after


def request(url_path: str, method: str = "GET", head: dict = None, params: dict = None, body: dict = None, AD = LADS):
    ADS = AD
    resp = requests.session().request(
        method=method,
        url=urljoin(ADS, url_path),
        headers=head,
        params=params,
        json=body,
    )
    if resp.status_code == 429:
        raise RateLimitError(resp.headers.get("Retry-After"))
    if resp.status_code >= 400:
        raise Exception(resp.text)
    return resp.json()

def convert_row(row, klist, tlist, alist):
    adgroupid = row['adGroupId']
    if adgroupid in klist:
        expressions = "keyword"
    elif adgroupid in tlist:
        expressions = "target"
    elif adgroupid in alist:
        expressions = "ads"
    else:
        expressions = "notmatch"
    return expressions


class SbGroup:
    def __init__(self, profile_id, campaignId: list = None):
        self.profile_id = profile_id
        self.portfolioId = campaignId
        self.re_url_path = "api/ad_manage/profiles/"
        self.upcreate_url_path = "api/ad_manage/sbgroups/p/updata/"
        self.kid_url_path = "api/ad_manage/sbkeywordsadgroupId/"
        self.tid_url_path = "api/ad_manage/sbtargetsadgroupId/"
        self.aid_url_path = "api/ad_manage/sbadsadgroupId/"
        self.heads = {'X-Token': "da4ab6bc5cbf1dfa"}
        self.refresh_token = self.get_refresh_token()
        self.lwa_client_id = AWS_LWA_CLIENT['lwa_client_id']
        self.lwa_client_secret = AWS_LWA_CLIENT['lwa_client_secret']
        self.AWS_CREDENTIALS = {
            'lwa_client_id': self.lwa_client_id,
            'lwa_client_secret': self.lwa_client_secret,
            'refresh_token': self.refresh_token,
            'profile_id': self.profile_id
        }

    def get_refresh_token(self):
        params = {'profile_id': self.profile_id}
        heads = self.heads
        url_path = self.re_url_path
        tem = request(url_path=url_path, head=heads, params=params)
        if tem.get('data') is not None:
            _ = tem.get('data')
            out = _[0].get('refresh_token')
        else:
            out = None
        return out

    def get_keyword_adgroupid_list(self):
        heads = self.heads
        url_path = self.kid_url_path
        data = []
        page = 1
        params = {'limit': 999, 'page': page}
        tem = request(url_path=url_path, head=heads, params=params)
        data.extend(tem.get('data'))
        while tem.get('is_next') is True:
            page += 1
            params = {'profile_id': self.profile_id, 'limit': 999, 'page': page}
            tem = request(url_path=url_path, head=heads, params=params)
            data.extend(tem.get('data'))
        df = pd.json_normalize(data)
        df.drop_duplicates(inplace=True)
        kglist = list(df.adGroupId)
        return kglist

    def get_target_adgroupid_list(self):
        heads = self.heads
        url_path = self.tid_url_path
        data = []
        page = 1
        params = {'limit': 999, 'page': page}
        tem = request(url_path=url_path, head=heads, params=params)
        data.extend(tem.get('data'))
        while tem.get('is_next') is True:
            page += 1
            params = {'profile_id': self.profile_id, 'limit': 999, 'page': page}
            tem = request(url_path=url_path, head=heads, params=params)
            data.extend(tem.get('data'))
        df = pd.json_normalize(data)
        df.drop_duplicates(inplace=True)
        tglist = list(df.adGroupId)
        return tglist

    def get_ads_adgroupid_list(self):
        heads = self.heads
        url_path = self.aid_url_path
        data = []
        page = 1
        params = {'limit': 999, 'page': page}
        tem = request(url_path=url_path, head=heads, params=params)
        data.extend(tem.get('data'))
        while tem.get('is_next') is True:
            page += 1
            params = {'profile_id': self.profile_id, 'limit': 999, 'page': page}
            tem = request(url_path=url_path, head=heads, params=params)
            data.extend(tem.get('data'))
        df = pd.json_normalize(data)
        df.drop_duplicates(inplace=True)
        alist = list(df.adGroupId)
        return alist

    def get_spgroup_data(self):
        tem = SBClient(**self.AWS_CREDENTIALS)
        list_group = tem.iter_adGroups(**{"includeExtendedDataFields": True,
                                          "stateFilter": {
                                              "include": ["ARCHIVED", "PAUSED", "ENABLED"]
                                          }})
        df_group = pd.json_normalize(list(list_group))
        return df_group
    # ----------------------------

    def dataconvert(self):
        df = self.get_spgroup_data()
        tem_t = self.get_target_adgroupid_list()
        tem_k = self.get_keyword_adgroupid_list()
        tem_a = self.get_ads_adgroupid_list()
        df['extendedData.creationDate'] = pd.to_datetime(
            df['extendedData.creationDate'], unit='ms').dt.strftime('%Y-%m-%d %H:%M:%S')
        df['extendedData.lastUpdateDate'] = pd.to_datetime(
            df['extendedData.lastUpdateDate'], unit='ms').dt.strftime('%Y-%m-%d %H:%M:%S')
        col = ['adGroupId', 'campaignId', 'name', 'state', 'extendedData.creationDate',
               'extendedData.lastUpdateDate', 'extendedData.servingStatus', 'extendedData.servingStatusDetails']
        old = pd.DataFrame(data=[], columns=col)
        tem = pd.concat([old, df]).reset_index()
        tem.drop(columns='index', inplace=True)
        tem.columns = [i.replace(".", "_") for i in tem.columns]
        tem.rename(columns={
            'name': 'adGroupName',
            'campaignId': 'campaign',
            'extendedData_creationDate': 'creationDateTime',
            'extendedData_lastUpdateDate': 'lastUpdateDateTime',
            'extendedData_servingStatus': 'servingStatus',
            'extendedData_servingStatusDetails': 'servingStatusDetails'
        }, inplace=True)
        tem['targetingType'] = tem.apply(convert_row, klist=tem_k, tlist=tem_t, alist=tem_a, axis=1)
        json_data = json.loads(tem.to_json(orient='records', force_ascii=False))
        return json_data

    def updata_create(self):
        body = self.dataconvert()
        heads = self.heads
        url_path = self.upcreate_url_path
        tem = request(url_path=url_path, head=heads, body=body, method="POST")
        return tem


if __name__ == '__main__':
    a = SbGroup(profile_id="3006125408623189")
    out = a.updata_create()
    # out = a.dataconvert()
    print(out)
