import requests
from urllib.parse import urljoin
from sync_amz_data.public.amz_ad_client import SBClient
from sync_amz_data.settings import AWS_LWA_CLIENT
import pandas as pd
import json
from sync_amz_data.tasks.datainsert.wg import LADS


class RateLimitError(Exception):
    def __init__(self, retry_after: str = None):
        self.retry_after = retry_after


def request(url_path: str, method: str = "GET", head: dict = None, params: dict = None, body: dict = None, AD = LADS):
    ADS = AD
    resp = requests.session().request(
        method=method,
        url=urljoin(ADS, url_path),
        headers=head,
        params=params,
        json=body,
    )
    if resp.status_code == 429:
        raise RateLimitError(resp.headers.get("Retry-After"))
    if resp.status_code >= 400:
        raise Exception(resp.text)
    return resp.json()


class SbKeyword:
    def __init__(self, profile_id, campaignId: list = None):
        self.profile_id = profile_id
        self.portfolioId = campaignId
        self.re_url_path = "api/ad_manage/profiles/"
        self.gid_url_path = "api/ad_manage/getsbadgroupId/"
        self.upcreate_url_path = "api/ad_manage/sbkeywords/p/updata/"
        self.heads = {'X-Token': "da4ab6bc5cbf1dfa"}
        self.refresh_token = self.get_refresh_token()
        self.lwa_client_id = AWS_LWA_CLIENT['lwa_client_id']
        self.lwa_client_secret = AWS_LWA_CLIENT['lwa_client_secret']
        self.AWS_CREDENTIALS = {
            'lwa_client_id': self.lwa_client_id,
            'lwa_client_secret': self.lwa_client_secret,
            'refresh_token': self.refresh_token,
            'profile_id': self.profile_id
        }

    def get_refresh_token(self):
        params = {'profile_id': self.profile_id}
        heads = self.heads
        url_path = self.re_url_path
        tem = request(url_path=url_path, head=heads, params=params)
        if tem.get('data') is not None:
            _ = tem.get('data')
            out = _[0].get('refresh_token')
        else:
            out = None
        return out

    def get_adgroup_data(self):
        heads = self.heads
        url_path = self.gid_url_path
        data = []
        page = 1
        params = {'profile_id': self.profile_id, 'limit': 999, 'page': page}
        tem = request(url_path=url_path, head=heads, params=params)
        data.extend(tem.get('data'))
        while tem.get('is_next') is True:
            page += 1
            params = {'profile_id': self.profile_id, 'limit': 999, 'page': page}
            tem = request(url_path=url_path, head=heads, params=params)
            data.extend(tem.get('data'))
        df = pd.json_normalize(data)
        out_list_id = df.adGroupId.to_list()
        return out_list_id

    def get_sbkeywords_data(self):
        tem = SBClient(**self.AWS_CREDENTIALS)
        _ = tem.iter_keywords()
        list_sbkeywords = [row for i in _ for row in i]
        df_sbkeywords = pd.json_normalize(list(list_sbkeywords))
        return df_sbkeywords

    def dataconvert(self):
        df = self.get_sbkeywords_data()
        col = ['keywordId', 'adGroupId', 'campaignId', 'keywordText',
               'nativeLanguageKeyword', 'matchType', 'state', 'bid']
        old = pd.DataFrame(data=[], columns=col)
        for i in col:
            if i not in df.columns:
                df['{}'.format(i)] = None
        df_new = df[col].copy()
        tem = pd.concat([old, df_new]).reset_index()
        tem.drop(columns='index', inplace=True)
        tem.columns = [i.replace(".", "_") for i in tem.columns]
        tem.rename(columns={
            'adGroupId': 'adGroup',
            'campaignId': 'campaign',
        }, inplace=True)
        json_data = json.loads(tem.to_json(orient='records', force_ascii=False))
        return json_data

    def updata_create(self):
        body = self.dataconvert()
        heads = self.heads
        url_path = self.upcreate_url_path
        tem = request(url_path=url_path, head=heads, body=body, method="POST")
        return tem


if __name__ == '__main__':
    a = SbKeyword(profile_id="3006125408623189")
    out = a.updata_create()
    # out = a.dataconvert()
    print(out)
