import requests
from urllib.parse import urljoin
from sync_amz_data.public.amz_ad_client import SBClient
from sync_amz_data.settings import AWS_LWA_CLIENT
import pandas as pd
import json
from sync_amz_data.tasks.datainsert.wg import LADS


class RateLimitError(Exception):
    def __init__(self, retry_after: str = None):
        self.retry_after = retry_after


def request(url_path: str, method: str = "GET", head: dict = None, params: dict = None, body: dict = None, AD = LADS):
    ADS = AD
    resp = requests.session().request(
        method=method,
        url=urljoin(ADS, url_path),
        headers=head,
        params=params,
        json=body,
    )
    if resp.status_code == 429:
        raise RateLimitError(resp.headers.get("Retry-After"))
    if resp.status_code >= 400:
        raise Exception(resp.text)
    return resp.json()


class SbNegtiveKeyword:
    def __init__(self, profile_id, campaignId: list = None):
        self.profile_id = profile_id
        self.portfolioId = campaignId
        self.re_url_path = "api/ad_manage/profiles/"
        self.upcreate_url_path = "api/ad_manage/sbnegativekeyword/p/updata/"
        self.heads = {'X-Token': "da4ab6bc5cbf1dfa"}
        self.refresh_token = self.get_refresh_token()
        self.lwa_client_id = AWS_LWA_CLIENT['lwa_client_id']
        self.lwa_client_secret = AWS_LWA_CLIENT['lwa_client_secret']
        self.AWS_CREDENTIALS = {
            'lwa_client_id': self.lwa_client_id,
            'lwa_client_secret': self.lwa_client_secret,
            'refresh_token': self.refresh_token,
            'profile_id': self.profile_id
        }

    def get_refresh_token(self):
        params = {'profile_id': self.profile_id}
        heads = self.heads
        url_path = self.re_url_path
        tem = request(url_path=url_path, head=heads, params=params)
        if tem.get('data') is not None:
            _ = tem.get('data')
            out = _[0].get('refresh_token')
        else:
            out = None
        return out

    def get_sbnegtivekeywords_data(self):
        tem = SBClient(**self.AWS_CREDENTIALS)
        _ = tem.get_negative_keyword()
        df_sbnegtivekeywords = pd.json_normalize(_)
        return df_sbnegtivekeywords

    def dataconvert(self):
        df = self.get_sbnegtivekeywords_data()
        df['profile'] = self.profile_id
        df.rename(columns={
            'adGroupId': 'adGroup',
            'campaignId': 'campaign',
            'extendedData_creationDateTime': 'creationDateTime'
        }, inplace=True)
        json_data = json.loads(df.to_json(orient='records', force_ascii=False))
        return json_data

    def updata_create(self):
        body = self.dataconvert()
        heads = self.heads
        url_path = self.upcreate_url_path
        tem = request(url_path=url_path, head=heads, body=body, method="POST")
        return tem


if __name__ == '__main__':
    a = SbNegtiveKeyword(profile_id="3006125408623189")
    # out = a.get_sbnegtivekeywords_data()
    out = a.updata_create()
    print(out)
