from sync_amz_data.DataTransform.Data_ETL import SB_ETL
import requests
from urllib.parse import urljoin
from sync_amz_data.settings import AWS_LWA_CLIENT
import pandas as pd
import json
from sync_amz_data.tasks.datainsert.wg import LADS
from sync_amz_data.public.amz_ad_client import SBClient
from typing import Literal


class RateLimitError(Exception):
    def __init__(self, retry_after: str = None):
        self.retry_after = retry_after


def request(url_path: str, method: str = "GET", head: dict = None, params: dict = None, body: dict = None, AD=LADS):
    ADS = AD
    resp = requests.session().request(
        method=method,
        url=urljoin(ADS, url_path),
        headers=head,
        params=params,
        json=body,
    )
    if resp.status_code == 429:
        raise RateLimitError(resp.headers.get("Retry-After"))
    if resp.status_code >= 400:
        raise Exception(resp.text)
    return resp.json()


class SbTargets:
    def __init__(self, profile_id):
        self.profile_id = profile_id
        self.re_url_path = "api/ad_manage/profiles/"
        self.gid_url_path = "api/ad_manage/getsbadgroupId/"
        self.upcreate_url_path = "api/ad_manage/sbtargets/p/updata/"
        self.heads = {'X-Token': "da4ab6bc5cbf1dfa"}
        self.refresh_token = self.get_refresh_token()
        self.lwa_client_id = AWS_LWA_CLIENT['lwa_client_id']
        self.lwa_client_secret = AWS_LWA_CLIENT['lwa_client_secret']
        self.AWS_CREDENTIALS = {
            'lwa_client_id': self.lwa_client_id,
            'lwa_client_secret': self.lwa_client_secret,
            'refresh_token': self.refresh_token,
            'profile_id': self.profile_id
        }

    def get_refresh_token(self):
        params = {'profile_id': self.profile_id}
        heads = self.heads
        url_path = self.re_url_path
        tem = request(url_path=url_path, head=heads, params=params)
        if tem.get('data') is not None:
            _ = tem.get('data')
            out = _[0].get('refresh_token')
        else:
            out = None
        return out

    def id_type_trans(self, df):
        id_columns = [i for i in df.columns if 'id' in i.lower() and 'bid' not in i.lower()]
        df[id_columns] = df[id_columns].fillna(-1).applymap(lambda x: round(int(x), 0)).astype("string")
        df[id_columns] = df[id_columns].applymap(lambda x: '' if x == '-1' else x)
        return df

    @classmethod
    def get_keyOvalue(cls, expressions, result: Literal['value', 'type']):
        if expressions not in [None, pd.NA, pd.NaT]:
            if len(expressions) > 1:
                return [i.get(result) for i in expressions]
            if len(expressions) == 1:
                sub_ = expressions[0].get(result)
                if sub_ is None:
                    return None
                elif type(sub_) == str:
                    return sub_
                elif type(sub_) == list:
                    return [i.get(result) for i in sub_]
                else:
                    return expressions
        else:
            return '-'

    def get_sbtargets_data(self):
        tem = SB_ETL(**self.AWS_CREDENTIALS)
        df = tem.targets_ETL()
        if len(df) > 0:
            df.rename(columns={'adgroupid': 'adGroup',
                               'campaignid': 'campaign',
                               'targetid': 'targetId',
                               'expressions_type': 'expression_type',
                               'expressions_value': 'expression_value',
                               'resolvedexpressions_type': 'resolvedExpression_type',
                               'resolvedexpressions_value': 'resolvedExpression_value'}, inplace=True)
            df.drop(columns=['expressions', 'resolvedexpressions'], inplace=True)
            df['profile'] = self.profile_id
        return df

    def get_adgroup_data(self):
        heads = self.heads
        url_path = self.gid_url_path
        data = []
        page = 1
        params = {'profile_id': self.profile_id, 'limit': 999, 'page': page}
        tem = request(url_path=url_path, head=heads, params=params)
        data.extend(tem.get('data'))
        while tem.get('is_next') is True:
            page += 1
            params = {'profile_id': self.profile_id, 'limit': 999, 'page': page}
            tem = request(url_path=url_path, head=heads, params=params)
            data.extend(tem.get('data'))
        df = pd.json_normalize(data)
        out_list_id = df.adGroupId.to_list()
        return out_list_id

    def get_sbtargets_data_new(self):
        tem = SBClient(**self.AWS_CREDENTIALS)
        adgroupids = self.get_adgroup_data()
        _ = tem.get_targets_groupid(adgroupIds=adgroupids)
        df = _.get('targets')
        df_targets = pd.json_normalize(df)
        if len(df_targets) > 0:
            df_targets = self.id_type_trans(df_targets)
            df_targets['expressions_type'] = df_targets['expressions'].map(
                lambda x: self.get_keyOvalue(x, 'type'))
            df_targets['expressions_value'] = df_targets['expressions'].map(
                lambda x: self.get_keyOvalue(x, 'value'))
            df_targets['resolvedExpressions_type'] = df_targets['resolvedExpressions'].map(
                lambda x: self.get_keyOvalue(x, 'type'))
            df_targets['resolvedExpressions_value'] = df_targets['resolvedExpressions'].map(
                lambda x: self.get_keyOvalue(x, 'value'))
            df_targets.rename(columns={'adGroupId': 'adGroup',
                                       'campaignId': 'campaign',
                                       'expressions_type': 'expression_type',
                                       'expressions_value': 'expression_value',
                                       'resolvedExpressions_type': 'resolvedExpression_type',
                                       'resolvedExpressions_value': 'resolvedExpression_value'}, inplace=True)
            df_targets.drop(columns=['expressions', 'resolvedExpressions'], inplace=True)
            df_targets['profile'] = self.profile_id
        return df_targets

    def updata_create(self):
        df_data = self.get_sbtargets_data_new()
        _ = df_data.to_json(orient='records', date_format='iso')
        body = json.loads(_)
        heads = self.heads
        url_path = self.upcreate_url_path
        tem = request(url_path=url_path, head=heads, body=body, method="POST")
        return tem


if __name__ == '__main__':
    a = SbTargets(profile_id="3006125408623189")
    # out = a.get_sbtargets_data_new()
    out = a.updata_create()
    print(out)
