import requests
from urllib.parse import urljoin
from sync_amz_data.public.amz_ad_client import SDClient
from sync_amz_data.settings import AWS_LWA_CLIENT
import pandas as pd
import json
from sync_amz_data.tasks.datainsert.wg import LADS


class RateLimitError(Exception):
    def __init__(self, retry_after: str = None):
        self.retry_after = retry_after


def request(url_path: str, method: str = "GET", head: dict = None, params: dict = None, body: dict = None, AD = LADS):
    ADS = AD
    resp = requests.session().request(
        method=method,
        url=urljoin(ADS, url_path),
        headers=head,
        params=params,
        json=body,
    )
    if resp.status_code == 429:
        raise RateLimitError(resp.headers.get("Retry-After"))
    if resp.status_code >= 400:
        raise Exception(resp.text)
    return resp.json()

class SpCampaign:
    def __init__(self, profile_id, portfolioId=None):
        self.profile_id = profile_id
        self.portfolioId = portfolioId
        self.re_url_path = "api/ad_manage/profiles/"
        self.upcreate_url_path = "api/ad_manage/sdcampaigns/p/updata/"
        self.heads = {'X-Token': "da4ab6bc5cbf1dfa"}
        self.refresh_token = self.get_refresh_token()
        self.lwa_client_id = AWS_LWA_CLIENT['lwa_client_id']
        self.lwa_client_secret = AWS_LWA_CLIENT['lwa_client_secret']
        self.AWS_CREDENTIALS = {
            'lwa_client_id': self.lwa_client_id,
            'lwa_client_secret': self.lwa_client_secret,
            'refresh_token': self.refresh_token,
            'profile_id': self.profile_id
        }

    def get_refresh_token(self):
        params = {'profile_id': self.profile_id}
        heads = self.heads
        url_path = self.re_url_path
        tem = request(url_path=url_path, head=heads, params=params)
        if tem.get('data') is not None:
            _ = tem.get('data')
            out = _[0].get('refresh_token')
        else:
            out = None
        return out

    def get_sdcampaign_data(self):
        tem = SDClient(**self.AWS_CREDENTIALS)
        list_campaigns = tem.get_campaigns_extended()
        df_campaigns = pd.json_normalize(list_campaigns)
        return df_campaigns

    def dataconvert(self):
        df = self.get_sdcampaign_data()
        df['profile_id'] = self.profile_id
        df['creationDate'] = pd.to_datetime(df['creationDate'], unit='ms').dt.strftime('%Y-%m-%d %H:%M:%S')
        df['lastUpdatedDate'] = pd.to_datetime(df['lastUpdatedDate'], unit='ms').dt.strftime('%Y-%m-%d %H:%M:%S')
        df['portfolioId'] = df['portfolioId'].apply(lambda x: str(int(x)) if not pd.isna(x) else x)
        df['startDate'] = pd.to_datetime(df['startDate'], format='%Y%m%d').dt.strftime('%Y-%m-%d')
        if 'endDate' in df.columns:
            df['endDate'] = pd.to_datetime(df['endDate'], format='%Y%m%d').dt.strftime('%Y-%m-%d')
        col = ['campaignId', 'tactic', 'name', 'startDate', 'budget', 'budgetType',
               'endDate', 'state', 'portfolioId', 'deliveryProfile', 'costType',
               'creationDate', 'lastUpdatedDate', 'profile_id', 'servingStatus',
               'ruleBasedBudget.isProcessing', 'ruleBasedBudget.applicableRuleName',
               'ruleBasedBudget.value', 'ruleBasedBudget.applicableRuleId']
        old = pd.DataFrame(data=[], columns=col)
        tem = pd.concat([old, df]).reset_index()
        tem.drop(columns='index', inplace=True)
        tem.columns = [i.replace(".", "_") for i in tem.columns]
        tem.rename(columns={
            'name': 'campaignName',
            'portfolioId': 'portfolio',
            'ruleBasedBudget_isProcessing': 'rbb_isProcessing',
            'ruleBasedBudget_applicableRuleName': 'rbb_appRuleName',
            'ruleBasedBudget_value': 'rbb_value',
            'ruleBasedBudget_applicableRuleId': 'rbb_appRuleId'}, inplace=True)
        json_data = json.loads(tem.to_json(orient='records', force_ascii=False))
        return json_data

    def updata_create(self):
        body = self.dataconvert()
        heads = self.heads
        url_path = self.upcreate_url_path
        tem = request(url_path=url_path, head=heads, body=body, method="POST")
        return tem


if __name__ == '__main__':
    a = SpCampaign(profile_id="3006125408623189")
    out = a.updata_create()
    # out = a.dataconvert()
    print(out)