import requests
from urllib.parse import urljoin
from sync_amz_data.public.amz_ad_client import SPClient
from sync_amz_data.settings import AWS_LWA_CLIENT
import pandas as pd
import json
from sync_amz_data.tasks.datainsert.wg import LADS

class RateLimitError(Exception):
    def __init__(self, retry_after: str = None):
        self.retry_after = retry_after


def request(url_path: str, method: str = "GET", head: dict = None, params: dict = None, body: dict = None, AD = LADS):
    ADS = AD
    resp = requests.session().request(
        method=method,
        url=urljoin(ADS, url_path),
        headers=head,
        params=params,
        json=body,
    )
    if resp.status_code == 429:
        raise RateLimitError(resp.headers.get("Retry-After"))
    if resp.status_code >= 400:
        raise Exception(resp.text)
    return resp.json()


def convert_row(row):
    l_t = None
    l_p = None
    l_r = None
    if len(row['dyp']) > 0:
        for i in row['dyp']:
            a = json.loads(i)
            if 'placement' in a:
                if a['placement'] == "PLACEMENT_TOP":
                    l_t = a['percentage']
                if a['placement'] == "PLACEMENT_PRODUCT_PAGE":
                    l_p = a['percentage']
                if a['placement'] == "PLACEMENT_REST_OF_SEARCH":
                    l_r = a['percentage']
    return l_t, l_p, l_r
# row['top'],row['product_page'],row['rest_of_search']

def parse_dict_list(lst):
    return [json.dumps(json_str) for json_str in lst]

class SpCampaign:
    def __init__(self, profile_id, portfolioId: list = None):
        self.profile_id = profile_id
        self.portfolioId = portfolioId
        self.re_url_path = "api/ad_manage/profiles/"
        self.upcreate_url_path = "api/ad_manage/spcampaigns/p/updata/?updata=1"
        self.heads = {'X-Token': "da4ab6bc5cbf1dfa"}
        self.refresh_token = self.get_refresh_token()
        self.lwa_client_id = AWS_LWA_CLIENT['lwa_client_id']
        self.lwa_client_secret = AWS_LWA_CLIENT['lwa_client_secret']
        self.AWS_CREDENTIALS = {
            'lwa_client_id': self.lwa_client_id,
            'lwa_client_secret': self.lwa_client_secret,
            'refresh_token': self.refresh_token,
            'profile_id': self.profile_id
        }

    def get_refresh_token(self):
        params = {'profile_id': self.profile_id}
        heads = self.heads
        url_path = self.re_url_path
        tem = request(url_path=url_path, head=heads, params=params)
        if tem.get('data') is not None:
            _ = tem.get('data')
            out = _[0].get('refresh_token')
        else:
            out = None
        return out

    def get_spcampaign_data(self):
        tem = SPClient(**self.AWS_CREDENTIALS)
        if self.portfolioId is None:
            list_campaigns = tem.iter_campaigns(**{"includeExtendedDataFields": True})
        else:
            list_campaigns = tem.iter_campaigns(**{"includeExtendedDataFields": True,
                                                   "portfolioIdFilter": {"include": self.portfolioId}
                                                   })
        df_campaigns = pd.json_normalize(list(list_campaigns))
        return df_campaigns

    def dataconvert(self):
        df = self.get_spcampaign_data()
        df['profile_id'] = self.profile_id
        df['extendedData.creationDateTime'] = pd.to_datetime(df['extendedData.creationDateTime']).dt.strftime(
            '%Y-%m-%d %H:%M:%S')
        df['extendedData.lastUpdateDateTime'] = pd.to_datetime(df['extendedData.lastUpdateDateTime']).dt.strftime(
            '%Y-%m-%d %H:%M:%S')
        col = ['campaignId', 'endDate', 'name', 'startDate', 'state', 'targetingType',
               'budget.budget', 'budget.budgetType', 'budget.effectiveBudget',
               'dynamicBidding.placementBidding', 'extendedData.lastUpdateDateTime',
               'dynamicBidding.strategy', 'portfolioId', 'profile_id',
               'extendedData.servingStatus', 'extendedData.servingStatusDetails',
               'extendedData.creationDateTime']
        old = pd.DataFrame(data=[], columns=col)
        tem = pd.concat([old, df]).reset_index()
        tem.drop(columns='index', inplace=True)
        tem.columns = [i.replace(".", "_") for i in tem.columns]
        tem['dyp'] = tem['dynamicBidding_placementBidding']
        tem.dyp = tem.dyp.apply(parse_dict_list)
        tem[['top', 'product_page', 'rest_of_search']] = tem.apply(convert_row, axis=1, result_type='expand')
        tem.drop(columns='dyp', inplace=True)
        tem.rename(columns={
            'name': 'campaignName',
            'portfolioId': 'portfolio',
            'dynamicBidding_placementBidding': 'dynBidPlacement',
            'dynamicBidding_strategy': 'dynBidStrategy',
            'budget_budgetType': 'budgetType',
            'budget_budget': 'budget',
            'budget_effectiveBudget': 'effectiveBudget',
            'extendedData_lastUpdateDateTime': 'lastUpdateDateTime',
            'extendedData_servingStatus': 'servingStatus',
            'extendedData_servingStatusDetails': 'servingStatusDetails',
            'extendedData_creationDateTime': 'creationDateTime'}, inplace=True)
        json_data = json.loads(tem.to_json(orient='records', force_ascii=False))
        return json_data

    def updata_create(self):
        body = self.dataconvert()
        heads = self.heads
        url_path = self.upcreate_url_path
        tem = request(url_path=url_path, head=heads, body=body, method="POST")
        return tem


if __name__ == '__main__':
    a = SpCampaign(profile_id="3006125408623189")
    out = a.updata_create()
    # out = a.dataconvert()
    print(out)
